@@ -463,10 +463,50 @@ sub _unique
 {
     my $self = shift;
     my @array = @_;
+    my @uarray = ();
+    my %hash;
 
-    my %hash = map {$_ => 1} @array;
+    for (my $i = 0; $i < @array; $i++) {
+        if (!defined($hash{$array[$i]})) {
+            $hash{$array[$i]} = 1;
+            push(@uarray, $array[$i]);
+        }
+    }
 
-    return keys %hash;
+    return @uarray;
+}
+
+sub lock
+{
+    my $self = shift;
+    my $conf = $self->{_config};
+
+    if (!defined($conf->{lock})) {
+        return 0;
+    }
+
+    if (!open($self->{lock}, "> $conf->{lock}[0]")) {
+        return 1;
+    }
+
+    flock($self->{lock}, 2);
+
+    return 0;
+}
+
+sub unlock
+{
+    my $self = shift;
+    my $conf = $self->{_config};
+
+    if (!defined($conf->{lock}) || !defined($self->{lock})) {
+        return 0;
+    }
+
+    close($self->{lock});
+    undef($self->{lock});
+
+    return 0;
 }
 
 =head1 SEE ALSO
