@@ -261,7 +261,7 @@ sub pre_modify
                             return -1;
                         } elsif ($str) {
                             foreach my $line (split(/\n/, $str)) {
-                                my ($rwattr, $value) = ($line =~ /(^[^:]*): *(.*)$/);
+                                my ($rwattr, $value) = ($line =~ /(^[^:]*): (.*)$/);
                                 if (!defined($replaced{$rwattr})) {
                                     @{$replaced{$rwattr}} = ();
                                 }
