@@ -168,7 +168,7 @@ sub _getStaticValue
                 $value = $static->{value}[$i]->{content};
             }
 
-            if ($value) {
+            if (defined($value) && $value ne '') {
                 push(@values, $value);
             }
         }
