@@ -188,10 +188,10 @@ sub search
             last;
         }
 
-        if ($retry >= $conf->{connection}->{retry}[0]) {
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
             return (LDAP_SERVER_DOWN, ());
         }
-        sleep $conf->{connection}->{interval}[0];
+        sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
     }
 
@@ -225,7 +225,7 @@ sub search
         }
 
         # search base entry
-        ($rc, $key, $entry) = $self->_baseSearch($obj, $pkeys, $objbase, 0, $deref, $sizeLim, $timeLim, undef);
+        ($rc, $key, $entry) = $self->_baseSearch($obj, $pkeys, $objbase, 0, $deref, $sizeLim, $timeLim, undef, $attrOnly, @attrs);
         if ($rc) {
             last;
         }
@@ -246,7 +246,7 @@ sub search
         my @entries;
         my $keys;
 
-        ($rc, $keys, @entries) = $self->_childSearch($obj, [@{$pkeys}, $key], $objbase, $objscope, $deref, $sizeLim, $timeLim, $filter);
+        ($rc, $keys, @entries) = $self->_childSearch($obj, [@{$pkeys}, $key], $objbase, $objscope, $deref, $sizeLim, $timeLim, $filter, $attrOnly, @attrs);
         push(@match_entries, @entries);
         $sizeLim = $sizeLim - @entries;
 
@@ -287,10 +287,10 @@ sub compare
             last;
         }
 
-        if ($retry >= $conf->{connection}->{retry}[0]) {
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
             return (LDAP_SERVER_DOWN, ());
         }
-        sleep $conf->{connection}->{interval}[0];
+        sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
     }
 
@@ -342,10 +342,10 @@ sub modify
             last;
         }
 
-        if ($retry >= $conf->{connection}->{retry}[0]) {
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
             return (LDAP_SERVER_DOWN, ());
         }
-        sleep $conf->{connection}->{interval}[0];
+        sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
     }
 
@@ -385,10 +385,10 @@ sub add
             last;
         }
 
-        if ($retry >= $conf->{connection}->{retry}[0]) {
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
             return (LDAP_SERVER_DOWN, ());
         }
-        sleep $conf->{connection}->{interval}[0];
+        sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
     }
 
@@ -465,10 +465,10 @@ sub delete
             last;
         }
 
-        if ($retry >= $conf->{connection}->{retry}[0]) {
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
             return (LDAP_SERVER_DOWN, ());
         }
-        sleep $conf->{connection}->{interval}[0];
+        sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
     }
 
@@ -778,13 +778,13 @@ sub _checkConfig
     }
 
     # connection
-    if (!defined($conf->{connection}->{retry})
-        || $conf->{connection}->{retry}[0] !~ /[0-9]*/) {
-        $conf->{connection}->{retry}[0] = 0;
+    if (!defined($conf->{connection}[0]->{retry})
+        || $conf->{connection}[0]->{retry}[0] !~ /[0-9]*/) {
+        $conf->{connection}[0]->{retry}[0] = 0;
     }
-    if (!defined($conf->{connection}->{interval})
-        || $conf->{connection}->{interval}[0] !~ /[0-9]*/) {
-        $conf->{connection}->{interval}[0] = 10;
+    if (!defined($conf->{connection}[0]->{interval})
+        || $conf->{connection}[0]->{interval}[0] !~ /[0-9]*/) {
+        $conf->{connection}[0]->{interval}[0] = 10;
     }
 
     # data object
@@ -1077,7 +1077,7 @@ sub _searchObjects
 sub _baseSearch
 {
     my $self = shift;
-    my ($obj, $pkeys, $base, $scope, $deref, $sizeLim, $timeLim, $filter) = @_;
+    my ($obj, $pkeys, $base, $scope, $deref, $sizeLim, $timeLim, $filter, $attrOnly, @attrs) = @_;
     my $oconf = $obj->{conf};
     my @pkeyarr = @{$pkeys};
 
@@ -1125,7 +1125,7 @@ sub _baseSearch
     }
     my $basefilter = Net::LDAP::Filter->new($filterStr);
 
-    my ($rc, $keys, @entries) = $self->_objSearch($obj, $pkeys, $pdn, -1, $basefilter);
+    my ($rc, $keys, @entries) = $self->_objSearch($obj, $pkeys, $pdn, -1, $basefilter, $attrOnly, @attrs);
     if ($rc) {
         return ($rc, undef, undef);
     }
@@ -1136,7 +1136,7 @@ sub _baseSearch
 sub _childSearch
 {
     my $self = shift;
-    my ($obj, $pkeys, $base, $scope, $deref, $sizeLim, $timeLim, $filter) = @_;
+    my ($obj, $pkeys, $base, $scope, $deref, $sizeLim, $timeLim, $filter, $attrOnly, @attrs) = @_;
     my $oconf = $obj->{conf};
     my @match_entries = ();
     my @match_keys = ();
@@ -1181,7 +1181,7 @@ sub _childSearch
             $cbase = $base;
         }
 
-        ($rc, $keys, @entries) = $self->_objSearch($objs[$i], $pkeys, $cbase, $sizeLim, $filter);
+        ($rc, $keys, @entries) = $self->_objSearch($objs[$i], $pkeys, $cbase, $sizeLim, $filter, $attrOnly, @attrs);
         push(@match_entries, @entries);
         push(@match_keys, @{$keys});
 
@@ -1201,7 +1201,7 @@ sub _childSearch
                 return (LDAP_OTHER, \@match_keys, @match_entries);
             }
 
-            ($rc, $keys, @entries) = $self->_childSearch($objs[$i], [@pkeyarr, ${$ckeys}[$j]], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter);
+            ($rc, $keys, @entries) = $self->_childSearch($objs[$i], [@pkeyarr, ${$ckeys}[$j]], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter, $attrOnly, @attrs);
             if ($rc) {
                 return ($rc, \@match_keys, @match_entries);
             }
@@ -1422,6 +1422,59 @@ sub _pwdFormat
     return $entry;
 }
 
+sub _writeUpdateLog
+{
+    my $self = shift;
+    my ($func, $file, $dn, @info) = @_;
+    my $conf = $self->{_config};
+    my $lock;
+    my $fd;
+    my $ldif;
+
+    if (!open($lock, ">$file.lock")) {
+        $self->log(level => 'crit', message => "Can't open lock file of update log: $file.lock");
+        return -1;
+    }
+
+    if (!open($fd, ">> $file")) {
+        $self->log(level => 'crit', message => "Can't open update log: $file");
+        return -1;
+    }
+
+    flock($lock, 2);
+    $ldif = "# ".strftime("%Y%m%d%H%M%S", localtime(time))."\ndn: $dn\nchangetype: $func\n";
+
+    if ($func eq 'modify') {
+        while (@info > 0) {
+            my $action = shift @info;
+            my $attr = shift @info;
+            my @values;
+
+            while (@info > 0 && $info[0] ne "ADD" && $info[0] ne "DELETE" && $info[0] ne "REPLACE") {
+                push(@values, shift @info);
+            }
+
+            $ldif = $ldif.lc($action).": $attr\n";
+            foreach my $val (@values) {
+                $ldif = "$ldif$attr: $val\n";
+            }
+            $ldif = "$ldif-\n";
+        }
+    } elsif ($func eq 'add') {
+        $ldif = "$ldif$info[0]"
+    } elsif ($func eq 'modrdn') {
+        $ldif = $ldif."newrdn: $info[0]\ndeleteoldrdn: $info[1]\n";
+    }
+
+    $ldif = encode('utf8', $ldif);
+    print $fd "$ldif\n";
+
+    close($fd);
+    close($lock);
+
+    return 0;
+}
+
 =head1 SEE ALSO
 
 L<LISM>
