@@ -738,9 +738,9 @@ sub _do_search
     my $class = $control ? ref(${$control}[0]) : undef;
     while (1) {
         if ($class) {
-            $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizeLimit => $sizeLim, timeLimit => $timeLim, filter => $filterStr, control => $control);
+            $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizelimit => $sizeLim, timelimit => $timeLim, filter => $filterStr, control => $control);
         } else {
-            $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizeLimit => $sizeLim, timeLimit => $timeLim, filter => $filterStr);
+            $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizelimit => $sizeLim, timelimit => $timeLim, filter => $filterStr);
         }
 
         if ($msg->code) {
